/*
 * Copy me if you can.
 * by 20h
 */

#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ndb.h>

void
usage(void)
{
	print("usage: [-i min] [-a max] [net!]host\n");
	exits("usage");
}

char *
search(Ndb *db, char *attr, uint min)
{
	Ndbs s;
	Ndbtuple *t, *nt;
	char val[128], *p;

	snprint(val, sizeof(val), "%ud", min);

	t = ndbsearch(db, &s, attr, val);
	while(t)
	{
		for(nt = t; nt; nt = nt->entry)
		{
			if(strcmp(nt->attr, "tcp") == 0)
			{
				p = strdup(nt->val);
				while(t)
				{
					ndbfree(t);
					t = ndbsnext(&s, attr, val);
				}
				return p;
			}
		}
		ndbfree(t);
		t = ndbsnext(&s, attr, val);
	}

	return nil;
}

void
main(int argc, char *argv[])
{
	uint min, max, sock;
	char *host, *t, *net, *port, *portname;
	Ndb *db;

	min = 1;
	max = 1024;
	host = "127.0.0.1";
	portname = nil;

	ARGBEGIN {
		case 'i':
			min = atoi(EARGF(usage()));
			break;
		case 'a':
			max = atoi(EARGF(usage()));
			break;
		default:
			usage();
	} ARGEND;

	if (argc > 1)
		usage();

	if(argc == 1)
		host = argv[0];

	db = ndbopen(nil);

	t = strchr(host, '!');
	if(t != nil)
	{
		*t = '\0';
		net = host;
		host = t + 1;
	} else {
		net = "tcp";
	}

	if(min > max)
	{
		fprint(2, "Invalid range. (min > max)\n");
		usage();
	}

	print("Starting scan on %s from %d to %d.\n", host, min, max);
	port = mallocz(6, 2);
	while(min <= max)
	{
		sprint(port, "%d\0", min);
		sock = dial(netmkaddr(host, net, port), 0, 0, 0);
		if(sock != -1)
		{
			if(db != nil)
				portname = search(db, "port", min);
			print("Port %d (%s) open.\n", min, portname ? portname : "unknown");
			close(sock);
		}
		min++;
	}

	free(port);
	ndbclose(db);
	exits(0);
}
